﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace AddFields
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void ValidFieldList()
        {
            XrMapLib.ShapeMapLayer lyr = Xr.Layers.GetLayerAsShapeMap("shp");
            XrMapLib.FieldSet fs = lyr.AttributeTable.FieldSet;

            int cntFields = fs.FieldCount;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                String FieldName = fs.GetFieldName(iField);
                XrMapLib.XrFieldTypeEnum fieldType = fs.GetFieldType(iField);
                int FieldLength = fs.GetFieldWidth(iField);
                int FieldDecimal = fs.GetFieldDecimal(iField);

                ListViewItem lvi = new ListViewItem();
                lvi.Text = FieldName;

                if (fieldType == XrMapLib.XrFieldTypeEnum.XrDouble || fieldType == XrMapLib.XrFieldTypeEnum.XrFloat)
                {
                    lvi.SubItems.Add("Float");
                }
                else if (fieldType == XrMapLib.XrFieldTypeEnum.XrInteger || fieldType == XrMapLib.XrFieldTypeEnum.XrShort || fieldType == XrMapLib.XrFieldTypeEnum.XrVeryShort)
                {
                    lvi.SubItems.Add("Integer");
                }
                if (fieldType == XrMapLib.XrFieldTypeEnum.XrString)
                {
                    lvi.SubItems.Add("String");
                }

                lvi.SubItems.Add(FieldLength.ToString());
                lvi.SubItems.Add(FieldDecimal.ToString());

                lvFieldList.Items.Add(lvi);
            }
        }

        private void btnSHP_Click(object sender, EventArgs e)
        {
            DialogResult DR = openFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                String FileName = openFileDialog.FileName;
                tbSHP.Text = FileName;

                Xr.Layers.RemoveAllLayer();
                Xr.Layers.AddShapeMapLayer("shp", FileName);
                Xr.WaitForAllConnections();

                ValidFieldList();
            }
        }

        private void btnAppend_Click(object sender, EventArgs e)
        {
            String FieldName = tbFieldName.Text;
            if (FieldName.Length == 0) return;
            if (cbFieldType.SelectedIndex == -1) return;
            String FieldType = cbFieldType.Text;
            int FieldLength;
            if (!Int32.TryParse(tbFieldLength.Text, out FieldLength)) return;
            int FieldDecimal;
            if (!Int32.TryParse(tbDecimalLength.Text, out FieldDecimal)) return;

            ListViewItem lvi = new ListViewItem();
            lvi.Text = FieldName;
            lvi.SubItems.Add(FieldType);
            lvi.SubItems.Add(FieldLength.ToString());
            lvi.SubItems.Add(FieldDecimal.ToString());

            lvFieldList.Items.Add(lvi);
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (lvFieldList.SelectedIndices.Count == 1)
            {
                int SelectIndex = lvFieldList.SelectedIndices[0];

                XrMapLib.ShapeMapLayer lyr = Xr.Layers.GetLayerAsShapeMap("shp");
                if (lyr == null) return;
                XrMapLib.FieldSet fs = lyr.AttributeTable.FieldSet;
                int cntFields = fs.FieldCount;

                if (SelectIndex < cntFields) return;
                
                lvFieldList.Items.RemoveAt(SelectIndex);
            }
        }

        private XrMapLib.ValueList GetAttributeValueList(XrMapLib.IAttributeRow row, int fieldsCount)
        {
            XrMapLib.ValueList values = new XrMapLib.ValueList();

            for (int iField = 0; iField < fieldsCount; ++iField)
            {
                String value = row.GetValueAsString(iField);
                values.AddValueByString(value);
            }

            return values;
        }

        private XrMapLib.Coordinate GetShapeCoord(XrMapLib.IShapeRow row)
        {
            XrMapLib.Coordinate result = new XrMapLib.Coordinate();
            XrMapLib.Coordinate coord = row.GetVertex(0);
            result.X = coord.X;
            result.Y = coord.Y;

            return result;
        }

        private XrMapLib.CoordinateListCollection GetShapeCoordList(XrMapLib.IShapeRow row)
        {
            XrMapLib.CoordinateListCollection CLC = new XrMapLib.CoordinateListCollection();

            int cntParts = row.PartCount;
            for (int iPart = 0; iPart < cntParts; ++iPart)
            {
                int cntVertices = row.GetVertexCountFromPart(iPart);
                XrMapLib.CoordinateList CL = new XrMapLib.CoordinateList();
                for (int iVertex = 0; iVertex < cntVertices; ++iVertex)
                {
                    XrMapLib.Coordinate coord = row.GetVertexFromPart(iPart, iVertex);
                    CL.Add(coord.X, coord.Y);
                }

                CLC.Add(CL);
            }

            return CLC;
        }

        private void AddNewFieldValues(XrMapLib.ValueList VL, int OriginalFieldCount)
        {
            int cntItems = lvFieldList.Items.Count;
            for (int iItem = OriginalFieldCount; iItem < cntItems; iItem++)
            {
                String FieldType = lvFieldList.Items[iItem].SubItems[1].Text;
                if (FieldType.CompareTo("String") == 0)
                {
                    VL.AddValueByString("");
                }
                else if (FieldType.CompareTo("Integer") == 0)
                {
                    VL.AddValueByInteger(0);
                }
                else if (FieldType.CompareTo("Float") == 0)
                {
                    VL.AddValueByFloat(0.0, 1);
                }
            }
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            XrMapLib.FieldSetDefine FSD = new XrMapLib.FieldSetDefine();

            int cntItems = lvFieldList.Items.Count;
            for (int i = 0; i < cntItems; i++)
            {
                ListViewItem item = lvFieldList.Items[i];
                String FieldName = item.Text;

                String FieldType = item.SubItems[1].Text;
                int FieldLength = Int32.Parse(item.SubItems[2].Text);
                int FieldDecimal = Int32.Parse(item.SubItems[3].Text);

                if (FieldType.CompareTo("String") == 0)
                {
                    FSD.AddStringField(FieldName, FieldLength);
                }
                else if (FieldType.CompareTo("Float") == 0)
                {
                    FSD.AddFloatField(FieldName, FieldLength, FieldDecimal);  
                }
                else if (FieldType.CompareTo("Integer") == 0)
                {
                    FSD.AddIntegerField(FieldName, FieldLength);  
                }
            }

            XrMapLib.ShapeMapLayer lyr = Xr.Layers.GetLayerAsShapeMap("shp");
            if (lyr == null)
            {
                MessageBox.Show("Error: Reading input shp file");
                return;
            }

            XrMapLib.Tool tool = new XrMapLib.Tool();
            XrMapLib.EsriShpFileHandler SFH = tool.CreateEsriShpFileHandler(tbOutSHP.Text, lyr.ShapeType, FSD);   
            if (tbOutSHP.Text.Length == 0 || SFH == null)   
            {   
                MessageBox.Show("Error: Creating shp file");   
                return;
            }

            bool bPointType = lyr.ShapeType == XrMapLib.XrShapeTypeEnum.XrPoint;
            int cntFields = lyr.AttributeTable.FieldSet.FieldCount;
            int cntRows = lyr.ShapeTable.RowCount;
            for (int iRow = 0; iRow < cntRows; ++iRow)
            {
                XrMapLib.ShapeRow SRow = lyr.ShapeTable.GetRow(iRow);
                XrMapLib.AttributeRow ARow = lyr.AttributeTable.GetRow(iRow);

                if (SRow.Load() && ARow.Load())
                {
                    XrMapLib.ValueList VL = GetAttributeValueList(ARow, cntFields);
                    AddNewFieldValues(VL, cntFields);

                    if (bPointType)
                    {
                        XrMapLib.Coordinate COORD = GetShapeCoord(SRow);
                        bool bOK = SFH.AddPoint(COORD.X, COORD.Y, VL);
                        if (!bOK)
                        {
                            MessageBox.Show("Error: Writing shp file");
                            return;
                        }
                    }
                    else
                    {
                        XrMapLib.CoordinateListCollection CLC = GetShapeCoordList(SRow);
                        bool bOK = SFH.AddMultiPart(CLC, VL);
                        if (!bOK)
                        {
                            MessageBox.Show("Error: Writing shp file");
                            return;
                        }
                    }


                    SRow.Unload();
                    ARow.Unload();
                }
                else
                {
                    MessageBox.Show("Error: Reading shp file");   
                    return;
                }

            }

            SFH.Close();

            MessageBox.Show("Completed");
        }

        private void btnOutSHP_Click(object sender, EventArgs e)
        {
            DialogResult DR = saveFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbOutSHP.Text = saveFileDialog.FileName;
            }
        }
    }
}
